import { Component, OnInit } from '@angular/core';
import { SelectItem, MessageService } from 'primeng/api';
import * as moment from 'moment';
import { ConversorCalendario } from 'src/app/calendario/conversor-calendario';
import { EnderecoService } from '../endereco/endereco.service';
import { Cliente } from '../cliente';
import { FormGroup, FormBuilder, Validators, FormControl, ValidationErrors } from '@angular/forms';
import { BreadcrumbService } from 'src/app/breadcrumb.service';
import { Router, ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { Endereco } from '../endereco/endereco';
import { ClientesService } from '../clientes.service';
import { HttpErrorResponse } from '@angular/common/http';
import { listAnimation } from 'src/app/animations';
import { DadosPessoais } from '../dados-pessoais';
import { Reacordatorio } from '../reacordatorio';
import { OauthService } from 'src/app/seguranca/oauth.service';
import { Page } from 'src/app/utils/page';
import { filter } from 'rxjs/operators';

declare var require: any;

@Component({
  selector: 'app-cadastro-cliente',
  templateUrl: './cadastro-cliente.component.html',
  styleUrls: ['./cadastro-cliente.component.css'],
  animations: [listAnimation]
})
export class CadastroClienteComponent implements OnInit {

  opcaoGeneros: SelectItem[];
  opcaoSimNao: SelectItem[];
  opcaoEstadoCivil: SelectItem[];
  opcaoComoConheceu: SelectItem[];
  uploadedFiles: any[] = [];
  clienteForm: FormGroup;
  dataAtual: Date;
  calendarLocale: any;
  cliente: Cliente;
  blockedPanel: boolean;
  enderecoValido: boolean;
  titulo: string;

  isProcessando: boolean;

  constructor(
    private formBuilder: FormBuilder,
    private breadcrumbService: BreadcrumbService,
    private messageService: MessageService,
    private enderecoService: EnderecoService,
    private clientesService: ClientesService,
    private activatedRoute: ActivatedRoute,
    private router: Router,
    private oauthService: OauthService,
  ) {
    this.cliente = new Cliente();

    const id = this.activatedRoute.snapshot.params['id'];

    this.breadcrumbService.setItems([
      { label: 'Clientes', routerLink: ['/clientes'], visible: true },
      { label: 'Novo', routerLink: ['/clientes/novo'], visible: !id },
      { label: 'Editar', routerLink: [`/clientes/${id}`], visible: id != undefined }
    ]);

    if (id) {
      this.clientesService.findById(id)
        .subscribe(
          (cliente: Cliente) => {
            if (cliente) {
              this.cliente = cliente;
              this.configurarFormulario();
              this.breadcrumbService.setItems([
                { label: 'Clientes', routerLink: ['/clientes'], visible: true },
                { label: this.cliente.nome, routerLink: [`/clientes/${id}`], visible: true }
              ]);
            } else {
              this.router.navigate(['/clientes']);
            }
          }
        )
    } else if (this.isAdministrador()) {
      //this.cliente = null;
    }

    this.titulo = (id) ? 'Editar' : 'Cadastrar';


  }

  isAdministrador() {
    return this.oauthService.checkPermission('ROLE_ADMINISTRADOR');
  }

  ngOnInit() {
    this.configurarFormulario();
    this.opcaoGeneros = [
      { label: 'Masculino', value: 'MASCULINO' },
      { label: 'Feminino', value: 'FEMININO' }
    ];
    this.opcaoSimNao = [
      { label: 'Sim', value: 'SIM' },
      { label: 'Não', value: 'NAO' }
    ];
    this.opcaoComoConheceu = [
      { label: 'Indicação', value: 'INDICACAO' },
      { label: 'Panfleto', value: 'PANFLETO' },
      { label: 'Facebook', value: 'FACEBOOK' },
      { label: 'Instagram', value: 'INSTAGRAM' },
      { label: 'Publicidade', value: 'PUBLICIDADE' }
    ];
    this.opcaoEstadoCivil = [
      { label: 'Solteiro', value: 'SOLTEIRO' },
      { label: 'Casado', value: 'CASADO' },
      { label: 'Separado', value: 'SEPARADO' },
      { label: 'Divorciado', value: 'DIVORCIADO' },
      { label: 'Viúvo', value: 'VIUVO' }
    ];
    this.dataAtual = moment().toDate();
    this.calendarLocale = ConversorCalendario.getCalendarLocale();
  }

  configurarFormulario() {
    if (!this.cliente.dadosPessoais) {
      this.cliente.dadosPessoais = new DadosPessoais();
    }

    this.clienteForm = this.formBuilder.group({
      idUsuario: [this.cliente.idUsuario],
      conta: [this.cliente.conta],
      nome: [this.cliente.nome, Validators.required],
      email: [this.cliente.email, [Validators.required, Validators.email], [this.validarEmailUnico.bind(this)]],
      login: [this.cliente.login],
      // login: [this.cliente.login, [Validators.required], [this.validarLoginUnico.bind(this)]],
      senha: [null],
      dadosPessoais: this.formBuilder.group({
        dataNascimento: [moment(this.cliente.dadosPessoais.dataNascimento).toDate(), Validators.required],
        genero: [this.cliente.dadosPessoais.genero, Validators.required],
        cpf: [this.cliente.dadosPessoais.cpf, [Validators.required, this.validarCPF.bind(this)], [this.validarCPFUnico.bind(this)]],
        rg: [this.cliente.dadosPessoais.rg],
        telefone: [this.cliente.dadosPessoais.telefone],
        celular: [this.cliente.dadosPessoais.celular],
        escolaridade: [this.cliente.dadosPessoais.escolaridade],
        profissao: [this.cliente.dadosPessoais.profissao],
        cargo: [this.cliente.dadosPessoais.cargo],
        estadoCivil: [this.cliente.dadosPessoais.estadoCivil],
        comoConheceu: [this.cliente.dadosPessoais.comoConheceu],
      }),
    });

    // if (this.isNovoCliente()) {
    //   this.clienteForm.get('senha').setValidators([Validators.required, Validators.minLength(3)]);
    // }

  }

  validarCPF(input: FormControl) {
    if (!input.value) {
      return null;
    }
    const CPF = require("cpf");
    if (CPF.isValid(input.value)) {
      return null;
    } else {
      return { cpfInvalido: true };
    }
  }

  onUpload(event) {
    this.mostrarMensagemSucesso('Upload realizado com sucesso!');
  }

  exibirDescricao(formControlGroup: string, formControlName: string): boolean {
    const group = this.clienteForm.get(formControlGroup);
    if (group) {
      const control = group.get(formControlName);
      if (control) {
        return (control.value == 'SIM') ? true : false;
      }
    }
    return false;
  }

  exibirOpcoesFemininas() {
    const group = this.clienteForm.get('dadosPessoais');
    if (group) {
      const control = group.get('genero');
      if (control) {
        return (control.value == 'FEMININO') ? true : false;
      }
    }
    return false;
  }

  salvar(gerarNovaConsulta: boolean) {
    this.copyPropertiesCliente();

    this.enderecoService.getEndereco()
      .subscribe(
        (endereco: Endereco) => {
          this.cliente.endereco = endereco;

          // salva apenas após recuperar o endereco
          this.isProcessando = true;
          const isCreate = this.cliente.idUsuario == null;
          if (isCreate) {
            this.clientesService.insert(this.cliente)
              .subscribe(
                (cliente: Cliente) => {
                  this.mostrarMensagemSucesso('Cliente cadastrado com sucesso!');
                  setTimeout(
                    () => {
                        this.router.navigate(['/clientes'])
                    }
                    , 500)
                    this.isProcessando = false;
                }
              )
          } else {
            this.clientesService.update(this.cliente, false)
              .subscribe(
                (cliente: Cliente) => {
                  this.mostrarMensagemSucesso('Cliente alterado com sucesso!');
                  setTimeout(
                    () => {
                      if (this.isUsuarioCliente()) { // se for um cliente logado
                        this.router.navigate(['/'])
                      } else {
                        this.router.navigate(['/clientes'])
                      }
                    }, 500
                  )
                  this.isProcessando = false;
                },
                (erro: HttpErrorResponse) => {
                  if (erro.status == 400) {
                    this.mostrarMensagemErro('Senha incorreta');
                  }
                  this.isProcessando = false;
                }
              )
          }

        }
      );

  }

  validarEmailUnico(input: FormControl): Observable<ValidationErrors> {
    return new Observable<ValidationErrors>(
      (observer) => {
        if (input.value == this.cliente.email) {
          observer.next(null);
          observer.complete();
        } else {
          this.clientesService.emailValidate(input.value)
            .subscribe(
              (value: Cliente) => {
                observer.next({ emailExistente: true })
                observer.complete();
              },
              (erro: HttpErrorResponse) => {
                if (erro.status == 404) {
                  observer.next(null);
                }
                observer.complete();
              }
            )
        }
      }
    );
  }

  validarLoginUnico(input: FormControl) {
    return new Observable(
      (observer) => {
        if (input.value == this.cliente.login) {
          observer.next(null);
          observer.complete();
        } else {
          this.clientesService.loginValidate(input.value)
            .subscribe(
              (value: Cliente) => {
                observer.next({ loginExistente: true })
                observer.complete();
              },
              (erro: HttpErrorResponse) => {
                if (erro.status == 404) {
                  observer.next(null);
                }
                observer.complete();
              }
            )
        }
      }
    );
  }

  validarCPFUnico(input: FormControl) {
    return new Observable(
      (observer) => {
        if (input.value == this.cliente.dadosPessoais.cpf) {
          observer.next(null);
          observer.complete();
        } else {
          this.clientesService.cpfValidate(input.value)
            .subscribe(
              (value: Cliente) => {
                observer.next({ cpfExistente: true })
                observer.complete();
              },
              (erro: HttpErrorResponse) => {
                if (erro.status == 404) {
                  observer.next(null);
                }
                observer.complete();
              }
            )
        }
      }
    );
  }

  mostrarMensagemErro(mensagem: string) {
    this.messageService.add({ severity: 'error', summary: 'Erro', detail: mensagem });
  }

  mostrarMensagemInfo(mensagem: string) {
    this.messageService.add({ severity: 'info', summary: 'Info', detail: mensagem });
  }

  mostrarMensagemSucesso(mensagem: string) {
    this.messageService.add({ severity: 'success', summary: 'Sucesso', detail: mensagem });
  }

  private copyPropertiesCliente() {
    this.cliente = this.clienteForm.value;
    this.cliente.status = 'ATIVO';
    this.cliente.permissoes = [{ idPermissao: 4, permissao:'ROLE_PESSOA' }];

    if (this.isNovoCliente()) { // se é novo cliente, replica o email para o campo login
      this.cliente.login = this.cliente.email;
    }
  }

  isEnderecoValido(event: boolean) {
    this.enderecoValido = event;
  }

  disableClienteForm() {
    return this.clienteForm.status != 'VALID' || !this.enderecoValido;
  }

  isNovoCliente() {
    return this.cliente.idUsuario == null;
  }

  isUsuarioCliente() {
    return this.oauthService.checkPermission('ROLE_CLIENTE');
  }

}
