import { NgModule, Injectable } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpRequest, HttpInterceptor, HttpHandler, HttpEvent, HTTP_INTERCEPTORS, HttpErrorResponse } from '@angular/common/http';
import { Observable, throwError } from 'rxjs';
import { OauthService } from '../oauth.service';
import { catchError, mergeMap, retry, map } from 'rxjs/operators';
import { JwtInterceptor } from '@auth0/angular-jwt';
import { Router } from '@angular/router';
import { MessageService } from 'primeng/api';

@Injectable()
export class HttpsRequestInterceptor implements HttpInterceptor {

  constructor(
    private oauth: OauthService,
    private jwtInterceptor: JwtInterceptor,
    private router: Router,
    private messageService: MessageService
  ) {
  }

  intercept(
    req: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    if (this.jwtInterceptor.isWhitelistedDomain(req) && !this.jwtInterceptor.isBlacklistedRoute(req)) {
      return next.handle(req).pipe(
        retry(2),
        catchError((erro: HttpErrorResponse) => {
          if (erro.status === 401 && erro.error.error === 'invalid_token') {
            return this.oauth.novoAccessToken().pipe(mergeMap(() => {
              return this.jwtInterceptor.intercept(req, next);
            }));
          } 
          else if (erro.status != 404) { 
            
            if (erro.status == 0) {
              this.mostrarMensagemInfo(`Houve um erro de conexão com a Internet. Por favor, verifique sua conexão e tente novamente.`);
            
            } else if (erro.status == 403) {
              this.mostrarMensagemInfo(`Você não possui permissão para efetuar esta operação.`);
            
            } else if(erro.status == 400){
              
              if(req.method == 'DELETE'){
                this.mostrarMensagemInfo(`Não é possível realizar esta operação pois existem registros associados a este.`);
              } else if(req.method == 'PUT' || req.method == 'POST'){
                this.mostrarMensagemInfo(`Uma inconsistência nos dados foi detectada. Por favor, verifique e tente novamente.`);
              } else {
                this.mostrarMensagemErro(`O sistema se comportou de maneira inesperada. Por favor, contate um de nossos desenvolvedores. Status: ${erro.status}`);
              }
            
            }else {
              this.mostrarMensagemErro(`O sistema se comportou de maneira inesperada. Por favor, contate um de nossos desenvolvedores. Status: ${erro.status}`);
            }

            console.error(`POSSÍVEL ERRO: ${erro.message}`)
          }
          return throwError(erro);
        })
      );
    } else if (this.oauth.urlOAuthToken === req.url) {
      return next.handle(req).pipe(
        retry(2),
        catchError((erro: HttpErrorResponse) => {
          if (erro.status === 401 && erro.error.error === 'invalid_token') {
            // if(this.oauth.isUsuarioLogado()){
            //   return next.handle(req);
            // } else {
            this.router.navigate(['/login']);
            return throwError(new Error('Sessão expirada.'));
            // }
          } else {
            return throwError(erro);
          }
        })
      );
    } else {
      return next.handle(req).pipe(
        retry(2),
        catchError((erro: HttpErrorResponse) => {
            
          if (erro.status == 0) {
            this.mostrarMensagemInfo(`Houve um erro de conexão com a Internet. Por favor, verifique sua conexão e tente novamente.`);
          
          } else if (erro.status == 403) {
            this.mostrarMensagemInfo(`Você não possui permissão para efetuar esta operação.`);
          
          } else if(erro.status == 400){
            
            if(req.method == 'DELETE'){
              this.mostrarMensagemInfo(`Não é possível realizar esta operação pois existem registros associados a este.`);
            } else if(req.method == 'PUT' || req.method == 'POST'){
              this.mostrarMensagemInfo(`Uma inconsistência nos dados foi detectada. Por favor, verifique e tente novamente.`);
            } else {
              this.mostrarMensagemErro(`O sistema se comportou de maneira inesperada. Por favor, contate um de nossos desenvolvedores. Status: ${erro.status}`);
            }
          
          }else {
            this.mostrarMensagemErro(`O sistema se comportou de maneira inesperada. Por favor, contate um de nossos desenvolvedores. Status: ${erro.status}`);
          }

          console.error(`POSSÍVEL ERRO: ${erro.message}`)

          return throwError(erro);
        })
      );
    }
  }

  private mostrarMensagemErro(mensagem: string) {
    this.messageService.add({ severity: 'error', summary: 'Erro', detail: mensagem });
  }

  private mostrarMensagemInfo(mensagem: string) {
    this.messageService.add({ severity: 'info', summary: 'Info', detail: mensagem });
  }

}

@NgModule({
  imports: [
    CommonModule
  ],
  providers: [
    JwtInterceptor, // Providing JwtInterceptor allow to inject JwtInterceptor manually into RefreshTokenInterceptor
    {
      provide: HTTP_INTERCEPTORS,
      useExisting: JwtInterceptor,
      multi: true
    },
    {
      provide: HTTP_INTERCEPTORS,
      useClass: HttpsRequestInterceptor,
      multi: true,
    },
  ],
  declarations: []
})
export class InterceptorModule { }
