import { Component, OnInit } from '@angular/core';
import { BreadcrumbService } from 'src/app/breadcrumb.service';
import { MessageService } from 'primeng/api';
import { Usuario } from '../usuario';
import { FormGroup, FormBuilder, Validators, FormControl, ValidationErrors } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { UsuariosService } from '../usuarios.service';
import { Observable } from 'rxjs';
import { HttpErrorResponse } from '@angular/common/http';
import { Permissao } from '../permissao';
import { DomSanitizer } from '@angular/platform-browser';
import { environment } from 'src/environments/environment';
import { OauthService } from 'src/app/seguranca/oauth.service';

@Component({
  selector: 'app-cadastro-administrador',
  templateUrl: './cadastro-administrador.component.html',
  styleUrls: ['./cadastro-administrador.component.css']
})
export class CadastroAdministradorComponent implements OnInit {

  titulo: string;
  administrador: Usuario;
  administradorForm: FormGroup;
  permissaoAdmin: Permissao;
  foto: any;

  constructor(
    private breadcrumbService: BreadcrumbService,
    private activatedRoute: ActivatedRoute,
    private administradoresService: UsuariosService,
    private oauthService: OauthService,
    private formBuilder: FormBuilder,
    private router: Router,
    private sanitizer: DomSanitizer,
    private messageService: MessageService
  ) {
    this.administrador = new Usuario();

    const id = this.activatedRoute.snapshot.params['id'];

    this.breadcrumbService.setItems([
      { label: 'Administradores', routerLink: ['/administradores'], visible: true },
      { label: 'Novo', routerLink: ['/administradores/novo'], visible: !id },
      { label: 'Editar', routerLink: [`/administradores/${id}`], visible: id != undefined }
    ]);

    if (id) {
      this.administradoresService.findById(id)
        .subscribe(
          (admin: Usuario) => {
            if (admin) {
              this.administrador = admin;

              this.breadcrumbService.setItems([
                { label: 'Administradores', routerLink: ['/administradores'], visible: true },
                { label: this.administrador.nome, routerLink: [`/administradores/${id}`], visible: true }
              ]);

              this.ngOnInit();
            } else {
              this.router.navigate(['/administradores']);
            }
          }
        )
    }

    this.titulo = (id) ? 'Editar' : 'Cadastrar';

    this.permissaoAdmin = { idPermissao: 1, permissao: 'ROLE_ADMINISTRADOR' };

  }

  ngOnInit() {

    const statusDefault = (this.administrador.status == 'ATIVO');
    this.administradorForm = this.formBuilder.group({
      idUsuario: [this.administrador.idUsuario],
      nome: [this.administrador.nome, Validators.required],
      email: [this.administrador.email, [Validators.required, Validators.email], [this.validarEmailUnico.bind(this)]],
      login: [this.administrador.login, [Validators.required], [this.validarLoginUnico.bind(this)]],
      senha: [null],
      status: [statusDefault, [Validators.required]],
    });

    if (this.isNovoUsuario()) {
      this.administradorForm.get('senha').setValidators([Validators.required, Validators.minLength(3)]);
    }

  }

  validarEmailUnico(input: FormControl): Observable<any> {
    return new Observable<ValidationErrors>(
      (observer) => {
        if (input.value == this.administrador.email) {
          observer.next(null);
          observer.complete();
        } else {
          this.administradoresService.emailValidate(input.value)
            .subscribe(
              (value: Usuario) => {
                observer.next({ emailExistente: true })
                observer.complete();
              },
              (error: HttpErrorResponse) => {
                if (error.status == 404) {
                  observer.next(null);
                }
                observer.complete();
              }
            )
        }
      }
    );
  }

  validarLoginUnico(input: FormControl) {
    return new Observable(
      (observer) => {
        if (input.value == this.administrador.login) {
          observer.next(null);
          observer.complete();
        } else {
          this.administradoresService.loginValidate(input.value)
            .subscribe(
              (value: Usuario) => {
                observer.next({ loginExistente: true })
                observer.complete();
              },
              (error: HttpErrorResponse) => {
                if (error.status == 404) {
                  observer.next(null);
                }
                observer.complete();
              }
            )
        }
      }
    );
  }

  salvar() {
    this.copyProperties();

    const isCreate = !this.administrador.idUsuario;

    if (isCreate) {
      this.administradoresService.insert(this.administrador)
        .subscribe(
          (usuario: Usuario) => {
            this.administrador = usuario;
            this.atualizarFoto();

            this.mostrarMensagemSucesso('Administrador(a) cadastrado com sucesso!');
            setTimeout(() => {
              this.router.navigate(['/administradores']);
            }, 500)
          }
        )
    } else {
      this.administradoresService.update(this.administrador, false)
        .subscribe(
          (usuario: Usuario) => {
            this.administrador = usuario;
            this.atualizarFoto();

            this.mostrarMensagemSucesso('Administrador(a) atualizado com sucesso!');
            setTimeout(() => {
              this.router.navigate(['/administradores']);
            }, 500)
          }
        )
    }
  }

  isNovoUsuario() {
    return this.administrador.idUsuario == null;
  }

  onSelect(event) {
    this.foto = event.files[0];
  }

  atualizarFoto() {
    if (this.foto) {
      const form = new FormData();
      form.append('foto', this.foto);
      this.administradoresService.updateFoto(form, this.administrador.idUsuario)
        .subscribe(
          (usuario: Usuario) => {
            // this.oauthService.novoAccessToken().subscribe();
          }
        );
    }
  }

  getURL() {
    if (this.foto) {
      const url = this.foto.objectURL.changingThisBreaksApplicationSecurity; // url temporaria
      return this.sanitizer.bypassSecurityTrustUrl(url);
    } else if (this.administrador.urlFoto) {
      return `${environment.apiUrl}/usuarios/foto/${this.administrador.urlFoto}`;
    } else {
      return 'assets/uploads/without-avatar.png';
    }
  }

  private copyProperties() {
    this.administrador = this.administradorForm.value;

    this.administrador.status = (this.administradorForm.value.status) ? 'ATIVO' : 'INATIVO';
    this.administrador.permissoes = [this.permissaoAdmin];
  }

  mostrarMensagemErro(mensagem: string) {
    this.messageService.add({ severity: 'error', summary: 'Erro', detail: mensagem });
  }

  mostrarMensagemInfo(mensagem: string) {
    this.messageService.add({ severity: 'info', summary: 'Info', detail: mensagem });
  }

  mostrarMensagemSucesso(mensagem: string) {
    this.messageService.add({ severity: 'success', summary: 'Sucesso', detail: mensagem });
  }

}
