package com.alessiojr.account.service;

import com.alessiojr.account.mail.PessoaMailer;
import com.alessiojr.account.repository.ContaRepository;
import com.alessiojr.account.model.Pessoa;
import com.alessiojr.account.model.PessoaDTO;
import com.alessiojr.account.model.Conta;
import com.alessiojr.account.repository.ContaRepository;
import com.alessiojr.common.enums.Status;
import com.alessiojr.location.model.Cidade;
import com.alessiojr.location.model.Endereco;
import com.alessiojr.location.model.Estado;
import com.alessiojr.location.repository.CidadeRepository;
import com.alessiojr.location.repository.EnderecoRepository;
import com.alessiojr.location.repository.EstadoRepository;
import com.alessiojr.security.enums.PermissionRoles;
import com.alessiojr.security.model.Permission;
import com.alessiojr.security.model.Usuario;
import com.alessiojr.security.repository.UsuarioRepository;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Random;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.security.crypto.bcrypt.BCrypt;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import com.alessiojr.account.repository.PessoaRepository;

@Service
public class PessoaService {

    @Autowired
    private UsuarioRepository<Usuario> usuariosPadrao;

    @Autowired
    private CidadeRepository cidades;

    @Autowired
    private EstadoRepository estados;

    @Autowired
    private PessoaRepository pessoas;

    @Autowired
    private EnderecoRepository enderecos;

    @Autowired
    private ContaRepository contas;

    @Autowired
    private PessoaMailer mailer;

    @Autowired
    private TransacaoService transacoes;

    public Page<Pessoa> findAllByNome(String nome, Integer pagina) {
        return this.pessoas.findAllByNome(nome, PageRequest.of(pagina, 10, Sort.by(Sort.Direction.ASC, "nome")));
    }

    public Optional<Pessoa> findByCPF(String cpf) {
        return this.pessoas.findByCPF(cpf);
    }

    public Optional<Pessoa> findByIdUsuario(Long idUsuario) {
        return this.pessoas.findByIdUsuario(idUsuario);
    }

    public Optional<Pessoa> findByLogin(String login) {
        return this.pessoas.findByLogin(login);
    }

    public Optional<Pessoa> findByEmail(String email) {
        return this.pessoas.findByEmail(email);
    }

    public Optional<Usuario> loginValidate(String login) {
        return this.usuariosPadrao.findByLogin(login);
    }

    public Optional<Usuario> emailValidate(String email) {
        return this.usuariosPadrao.findByEmail(email);
    }

//    public List<Object> countPessoasByEstados() {
//        return this.pessoas.countPessoasByEstados();
//    }
//
//    public List<Object> countPessoasByGenero() {
//        return this.pessoas.countPessoasByGenero();
//    }
//
//    public List<Object> countPessoasByComoConheceu() {
//        return this.pessoas.countPessoasByComoConheceu();
//    }
//
//    public Integer getMediaIdade() {
//        return this.pessoas.getMediaIdade();
//    }

    public void deleteById(Long idUsuario) {
        try {
            this.pessoas.deleteById(idUsuario);
        } catch (Exception e) {
            Optional<Pessoa> op = this.findByIdUsuario(idUsuario);
            op.get().setStatus(Status.INATIVO);

            this.pessoas.save(op.get());
        }
    }

    public Pessoa insert(Pessoa pessoa) throws Exception {
        String senha = pessoa.getSenha();
        boolean possuiSenha = senha != null;

        if (!possuiSenha) {
            senha = this.generatePassword();
        }

        // salvar a senha utilizando o metodo bcrypt para realizar a criptografia
        pessoa.setSenha(new BCryptPasswordEncoder().encode(senha));

        Conta c = new Conta();
        c.setSaldo(BigDecimal.ZERO);
        c = this.contas.save(c);
        pessoa.setConta(c);

        Set<Permission> permissoes = new HashSet<>();
        Permission p = new Permission();
        p.setIdPermissao(Long.valueOf(4));
        p.setPermissao(PermissionRoles.ROLE_USUARIO);
        permissoes.add(p);
        pessoa.setPermissoes(permissoes);

        pessoa.setStatus(Status.ATIVO);

        pessoa.setEndereco(getEndereco(pessoa.getEndereco()));
 
        pessoa = this.pessoas.save(pessoa); // salva primeiro para depois enviar o email

        if (possuiSenha) {
            this.mailer.enviarConfirmacaoCadastro(pessoa);
        } else {
            this.mailer.enviarConfirmacaoCadastro(pessoa, senha);
        }

        return pessoa;
    }

    public Optional<Pessoa> update(PessoaDTO pessoa, Pessoa existente, Boolean alterarSenha) {
        if (alterarSenha) {
            if (!BCrypt.checkpw(pessoa.getSenha(), existente.getSenha())) {// compara senhas antigas
                return Optional.empty(); // se senhas diferentes
            }
            pessoa.setSenha(new BCryptPasswordEncoder().encode(pessoa.getNovaSenha())); // altera a senha
            BeanUtils.copyProperties(pessoa, existente, "idUsuario", "novaSenha", "permissoes", "conta", "anamnese");

        } else {
            BeanUtils.copyProperties(pessoa, existente, "idUsuario", "senha", "novaSenha", "permissoes", "conta", "anamnese");
        }

        Conta aux = this.contas.findById(pessoa.getConta().getIdConta()).get();
        existente.setConta(aux);

        existente.setEndereco(getEndereco(pessoa.getEndereco()));

        return Optional.ofNullable(this.pessoas.save(existente));
    }

    private Cidade getCidade(Endereco end) {
        Cidade cidade = end.getCidade();
        Optional<Cidade> op = this.cidades.findByNomeLike(cidade.getNome());
        if (op.isPresent()) {
            return op.get();
        } else {
            Cidade nova = new Cidade();
            BeanUtils.copyProperties(cidade, nova, "idCidade");
            return this.cidades.save(nova);
        }
      
    }

    private Estado getEstado(Endereco end) {
        Estado estado = end.getCidade().getEstado();

        Optional<Estado> op = this.estados.findByUf(estado.getUf());
        if (op.isPresent()) {
            return op.get();
        } else {
            Estado novo = new Estado();
            BeanUtils.copyProperties(estado, novo, "idEstado");
            return (Estado)this.estados.save(novo);
        }
    }

    private Endereco getEndereco(Endereco temp) {
        temp.getCidade().setEstado(getEstado(temp));
        temp.setCidade(getCidade(temp));

        Endereco endereco;
        if (temp.getIdEndereco() != null) {
            endereco = this.enderecos.findById(temp.getIdEndereco()).get();
        } else {
            endereco = new Endereco();
        }

        BeanUtils.copyProperties(temp, endereco, "idEndereco");
        return this.enderecos.save(endereco);
    }

    private String generatePassword() {
        String senha = "";
        Random r = new Random();

        for (int i = 0; i < 6; i++) {
            if (i % 2 == 0) {
                senha += Character.toString((char) (97 + r.nextInt(26))); // gera letras minusculas
            } else {
                senha += Character.toString((char) (48 + r.nextInt(10))); // gera numeros
            }
        }

        return senha;
    }

}
