package com.alessiojr.account.service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import com.alessiojr.account.enums.CategoriaTransacao;
import com.alessiojr.account.enums.NomeTipoTransacao;
import com.alessiojr.account.model.Conta;
import com.alessiojr.account.model.TipoTransacao;
import com.alessiojr.account.model.Transacao;
import com.alessiojr.account.repository.ContaRepository;
import com.alessiojr.account.repository.TipoTransacaoRepository;
import com.alessiojr.account.repository.TransacaoRepository;
import java.util.ArrayList;
import java.util.Optional;
import java.util.Set;

@Service
public class TransacaoService {

    @Autowired
    private TransacaoRepository transacoes;

    @Autowired
    private TipoTransacaoRepository tiposTransacao;

    @Autowired
    private ContaRepository contas;

    public Page<Transacao> findAll(Integer pagina) {
        return this.transacoes.findAll(PageRequest.of(pagina, 10, Sort.by(Sort.Direction.DESC, "dataHora")));
    }

    public Page<Transacao> findByConta(Long conta, Integer pagina) {
        return this.transacoes.findByConta(conta, PageRequest.of(pagina, 10, Sort.by(Sort.Direction.DESC, "dataHora")));
    }

    public Page<Transacao> findByDataHoraBetween(LocalDate inicio, LocalDate fim, Integer pagina) {
        return this.transacoes.findByDataHoraBetween(inicio.atStartOfDay(), fim.atTime(23, 59, 59), PageRequest.of(pagina, 10, Sort.by(Sort.Direction.DESC, "dataHora")));
    }

    public Page<Transacao> findByDataHoraAfter(LocalDate dataHora, Integer pagina) {
        return this.transacoes.findByDataHoraAfter(dataHora.atStartOfDay(), PageRequest.of(pagina, 10, Sort.by(Sort.Direction.DESC, "dataHora")));
    }

    public Page<Transacao> findByDataHoraBefore(LocalDate dataHora, Integer pagina) {
        return this.transacoes.findByDataHoraBefore(dataHora.atStartOfDay(), PageRequest.of(pagina, 10, Sort.by(Sort.Direction.DESC, "dataHora")));
    }

    public BigDecimal getFaturamentoByMesAndAno(Integer mes, Integer ano) {
        return this.transacoes.getFaturamentoByMesAndAno(mes, ano);
    }

    public BigDecimal getFaturamentoByContaAndMesAndAno(Long conta, Integer mes, Integer ano) {
        return this.transacoes.getFaturamentoByContaAndMesAndAno(conta, mes, ano);
    }

    public Set<Transacao> findAllByConsulta(Long idConsulta){
        return this.transacoes.findAllByConsulta(idConsulta);
    }
    
    public void deleteAllByConsulta(Long idConsulta){
        this.transacoes.findAllByConsulta(idConsulta).forEach(t -> this.deleteTransacao(t.getIdTransacao()));
    }
    
    public void deleteTransacao(Long idTransacao){
        Optional<Transacao> op = this.transacoes.findById(idTransacao);
        if(op.isPresent()){
            Transacao t = op.get();
            // recalcular o saldo 
            if(t.getTipo().getTipo() == CategoriaTransacao.CREDITO){
                t.getConta().setSaldo( t.getConta().getSaldo().subtract(t.getValor()) );
            } else {
                t.getConta().setSaldo( t.getConta().getSaldo().add(t.getValor()) );
            }
            t.getConta().getTransacoes().remove(t); // remove a transacao
            
            this.contas.save(t.getConta());
        }
    }

    private BigDecimal getValorBrutoPorcentagem(BigDecimal valor, Float porcentagem) {
        // retorna o valor calculado baseado na porcentagem
        return valor.multiply(BigDecimal.valueOf(porcentagem)).divide(BigDecimal.valueOf(100));

    }

    private boolean possuiCreditoValido(Conta conta, BigDecimal valorConsulta) {
        return conta.getSaldo().compareTo(valorConsulta) >= 0;
    }

    private void saveTransacao(NomeTipoTransacao nome, String descricao, BigDecimal valor, Conta conta) {
        Transacao transacao = new Transacao();
        transacao.setDataHora(LocalDateTime.now());
        transacao.setTipo(this.getTipoTransacao(nome));
        transacao.setDescricao(descricao);
        transacao.setValor(valor);
        transacao.setConta(conta);

        transacao = this.transacoes.save(transacao);

        if (conta.getTransacoes() == null) {
            conta.setTransacoes(new ArrayList<>());
        }

        conta.getTransacoes().add(transacao);

        if (transacao.getTipo().getTipo() == CategoriaTransacao.CREDITO) {
            conta.setSaldo(conta.getSaldo().add(transacao.getValor()));
        } else if (transacao.getTipo().getTipo() == CategoriaTransacao.DEBITO) {
            conta.setSaldo(conta.getSaldo().subtract(transacao.getValor()));
        }

        this.contas.save(conta);
    }

    private void saveTransacao(NomeTipoTransacao nome, LocalDateTime data, String descricao, BigDecimal valor, Conta conta) {
        Transacao transacao = new Transacao();
        transacao.setDataHora(data);
        transacao.setTipo(this.getTipoTransacao(nome));
        transacao.setDescricao(descricao);
        transacao.setValor(valor);
        transacao.setConta(conta);

        transacao = this.transacoes.save(transacao);

        if (conta.getTransacoes() == null) {
            conta.setTransacoes(new ArrayList<>());
        }

        conta.getTransacoes().add(transacao);

        if (transacao.getTipo().getTipo() == CategoriaTransacao.CREDITO) {
            conta.setSaldo(conta.getSaldo().add(transacao.getValor()));
        } else if (transacao.getTipo().getTipo() == CategoriaTransacao.DEBITO) {
            conta.setSaldo(conta.getSaldo().subtract(transacao.getValor()));
        }

        this.contas.save(conta);
    }

    public TipoTransacao getTipoTransacao(NomeTipoTransacao nome) {
        return this.tiposTransacao.findById(nome).get();
    }

}
