package com.alessiojr.drDebug.resource;

import com.alessiojr.drDebug.model.GameNivel;
import com.alessiojr.drDebug.service.GameNivelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.sql.SQLIntegrityConstraintViolationException;
import java.util.List;
import java.util.Optional;
import javax.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.UnexpectedRollbackException;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@CrossOrigin
@RestController
@RequestMapping("/niveisGame")
@Api(value = "/drdebug/niveisGame", tags = "Nivel Game Controller", description = "Operação sobre Entidade Niveis de Game")
public class GameNivelController {

    private static final Logger log = LoggerFactory.getLogger(GameNivelController.class);

    @Autowired
    private GameNivelService gniveis;

    // Get - read all resorce
    @ApiOperation(value = "Ver a lista dos niveis dos games", response = List.class)
    @GetMapping(params = {"nome", "pagina"})
    public Page<GameNivel> findAll(@RequestParam String nome, @RequestParam Integer pagina) {
        return this.gniveis.findByNomeContaining(nome, pagina);
    }

    // Get - Read a resource
    @GetMapping("/{id}")
    @ApiOperation(value = "Pesquisa um nivel de um game pelo id")
    public ResponseEntity<GameNivel> findById(@PathVariable Long id) {
        log.info("findById " + id);
        Optional<GameNivel> nivel = gniveis.findById(id);

        if (!nivel.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(nivel.get());
    }

    // Get - Read a resource
    @DeleteMapping("/{id}")
    @ApiOperation(value = "Apaga um nivel de um game pelo um id")
    public ResponseEntity<GameNivel> delete(@PathVariable Long id) {
        log.info("Passei no Delete /niveisGame" + id);
        Optional<GameNivel> niveis = gniveis.findById(id);
        if (!niveis.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            gniveis.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.status(200).build();
    }

    // PUT - Update an existing resource
    @PutMapping(value = "/{id}")
    @ApiOperation(value = "Altera os atributos de um nivel de um game", response = GameNivel.class)
    @ApiResponses(value = { @ApiResponse(code = 200, message = "Successfully"),
            @ApiResponse(code = 424, message = "Houve um problema com o game"), })
    @ResponseStatus(code = HttpStatus.FAILED_DEPENDENCY, reason = "Houve um problema com o game")
    public ResponseEntity<GameNivel> update(@RequestBody @Valid GameNivel nivel, @PathVariable Long id) {

        Optional<GameNivel> optional = gniveis.findById(id);

        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        GameNivel existente = optional.get();

        try {
            return ResponseEntity.ok(gniveis.update(nivel, existente));
        } catch (UnexpectedRollbackException ex) {
            if (ex.getMostSpecificCause() instanceof SQLIntegrityConstraintViolationException) {
                if (ex.getMessage().contains("FK_GameNivel_GameNivelgame")) {
                    return ResponseEntity.status(HttpStatus.FAILED_DEPENDENCY).build();
                }
            }
        }

        return ResponseEntity.status(500).build();
    }

    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Cria um novo nivel de game", response = GameNivel.class)
    @ApiResponses(value = { @ApiResponse(code = 201, message = "Successfully Created"),
            @ApiResponse(code = 400, message = "Já existe uma pessoa para este com a mesma descrição"),
            @ApiResponse(code = 401, message = "You are not authorized to view the resource"),
            @ApiResponse(code = 403, message = "Accessing the resource you were trying to reach is forbidden"),
            @ApiResponse(code = 404, message = "The resource you were trying to reach is not found") })
    public ResponseEntity<GameNivel> create(@ApiParam(value = "") @Valid @RequestBody GameNivel gameNivel) {

        return ResponseEntity.status(HttpStatus.CREATED).body(gniveis.insert(gameNivel));

    }

}

// 200 - SUCESS
// 404 - RESOURCE NOT FOUND
// 400 - BAD REQUEST
// 201 - CREATED
// 401 - UNAUTHORIZED
// 415 - UNSUPPORTED TYPE - Representation not supported for the resource
// 500 - SERVER ERROR
// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
// https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg
// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
 * GET /device-management/devices : Get all devices POST
 * /device-management/devices : Create a new device
 * 
 * GET /device-management/devices/{id} : Get the device information identified
 * by "id" PUT /device-management/devices/{id} : Update the device information
 * identified by "id" DELETE /device-management/devices/{id} : Delete device by
 * "id"
 */
