package com.alessiojr.estoque.resource;

import java.util.List;
import java.util.Optional;

import javax.validation.Valid;

import com.alessiojr.estoque.model.Compra;
import com.alessiojr.estoque.service.CompraService;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/**
 *
 * @author Abner Kaizer
 */
@CrossOrigin
@RestController
@RequestMapping("/estoque/compras")
@Api(value = "/estoque/compras", tags = "Compra Controller", description = "Operações sobre a Entidade Compra")
public class CompraController {

    private static final Logger log = LoggerFactory.getLogger(CompraController.class);

    @Autowired
    private CompraService compras;

    //GET - Read all resource
    @ApiOperation(value = "View a list of available compras", response = List.class)
    public Page<Compra> findAll(
            @RequestParam Long compraid,
            @RequestParam Integer pagina) {
        return this.compras.findByIdContaining(compraid, pagina);
    }

    @ApiOperation(value = "View a list of available compras", response = List.class)
    @GetMapping(params = {"pagina"})
    public Page<Compra> findAll(
            @RequestParam Integer pagina) {
        return this.compras.findAll(pagina);
    }

    //GET - Read a resource
    @GetMapping("/{compraid}")
    public ResponseEntity<Compra> findById(@PathVariable Long compraid) {
        log.info("Passei no /estoque/compras/" + compraid);
        Optional<Compra> compra = compras.findById(compraid);

        if (!compra.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(compra.get());
    }

    @DeleteMapping("/{compraid}")
//    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity delete(@PathVariable Long compraid) {
        log.info("Passei no Delete /estoque/compras/" + compraid);
        Optional<Compra> parceiro = compras.findById(compraid);
        if (!parceiro.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            compras.deleteById(compraid);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }

    //PUT - Update an existing resource
    @PutMapping("/{compraid}")
//    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Compra> update(@RequestBody @Valid Compra compra, @PathVariable Long compraid) {
        Optional<Compra> optional = compras.findById(compraid);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        Compra existente = optional.get();
        return ResponseEntity.ok(compras.update(compra, existente));
    }

    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Create a Single  compra", response = Compra.class)
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Successfully Created"),
        @ApiResponse(code = 400, message = "Já existe uma compra com a mesma descrição"),
        @ApiResponse(code = 401, message = "You are not authorized to view the resource"),
        @ApiResponse(code = 403, message = "Accessing the resource you were trying to reach is forbidden"),
        @ApiResponse(code = 404, message = "The resource you were trying to reach is not found")
    })
    public ResponseEntity<Compra> create(
            @ApiParam(value = "Compra e seus atributos") @Valid @RequestBody Compra compra) {
        return ResponseEntity
                .status(HttpStatus.CREATED)
                .body(compras.insert(compra));
    }
}
//200 - SUCESS
//404 - RESOURCE NOT FOUND
//400 - BAD REQUEST
//201 - CREATED
//401 - UNAUTHORIZED
//415 - UNSUPPORTED TYPE - Representation not supported for the resource
//418 - I AM A TEAPOT
//500 - SERVER ERROR

// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
//https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg
// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
GET 	/device-management/devices : Get all devices
POST 	/device-management/devices : Create a new device

GET 	/device-management/devices/{id} : Get the device information identified by "id"
PUT 	/device-management/devices/{id} : Update the device information identified by "id"
DELETE	/device-management/devices/{id} : Delete device by "id"*/
