package com.alessiojr.freeTests.resource;

import com.alessiojr.freeTests.enums.PessoaTesteOrderBy;
import com.alessiojr.freeTests.model.PessoaTeste;
import java.util.List;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.util.Optional;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseStatus;
import com.alessiojr.freeTests.service.PessoasTesteService;
import java.sql.SQLIntegrityConstraintViolationException;
import org.aspectj.bridge.MessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.UnexpectedRollbackException;
import org.springframework.web.bind.annotation.RequestParam;

@CrossOrigin
@RestController
@RequestMapping("/testes/pessoas")
@Api(value = "/testes/pessoas", tags = "Pessoa Teste", description = "Recurso para teste de API e Swagger")
public class PessoaTesteResource {

    private static final Logger LOG = LoggerFactory.getLogger(PessoaTesteResource.class);

    @Autowired
    private PessoasTesteService pessoas;
   
    //GET - Read all resource
    @ApiOperation(value = "View a list of available pessoas")
    @GetMapping
    public Page<PessoaTeste> findAll(
            @RequestParam(defaultValue = "%") Optional<String> filtro,
            @RequestParam(defaultValue = "0") Optional<Integer> pag,
            @RequestParam(defaultValue = "10") Optional<Integer> tam, 
            @RequestParam(defaultValue = "ID") PessoaTesteOrderBy orderBy,
            @RequestParam(defaultValue = "ASC") Sort.Direction orderDir
        ) 
    {
        return this.pessoas.findBySearchTerm(filtro.get(), pag.get(), tam.get(), orderBy, orderDir);
    }
    
    @PostMapping
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Create a Single Pessoa", response = PessoaTeste.class)
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Successfully Created"),
        @ApiResponse(code = 400, message = "Já existe uma pessoa para este com a mesma descrição"),
        @ApiResponse(code = 401, message = "You are not authorized to view the resource"),
        @ApiResponse(code = 403, message = "Accessing the resource you were trying to reach is forbidden"),
        @ApiResponse(code = 404, message = "The resource you were trying to reach is not found")
    })
    public ResponseEntity<PessoaTeste> create(
            @ApiParam(value = "Pessoas com elementos Nome e CPf") @Valid @RequestBody PessoaTeste pessoa) {
        return ResponseEntity
                .status(HttpStatus.CREATED)
                .body(pessoas.insert(pessoa));
    }
    
    //GET - Read a resource
    @GetMapping("/{id}")
    public ResponseEntity<PessoaTeste> findById(@PathVariable Long id) {
        
        Optional<PessoaTeste> pessoa = pessoas.findById(id);

       
        if (!pessoa.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(pessoa.get());
    }

    @DeleteMapping("/{id}")
//    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    @ApiResponses(value = {
            @ApiResponse(code = 404, message = "Pessoa Teste não Encontrada")
    })
    public ResponseEntity delete(@PathVariable Long id) {
        LOG.debug("Delete /testes/pessoas/" + id);
        Optional<PessoaTeste> parceiro = pessoas.findById(id);
        if (!parceiro.isPresent()) {
            return ResponseEntity
                    .status(HttpStatus.NOT_FOUND)
                    .body("Pessoa Teste não Encontrada");
        }

        try {
            pessoas.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }

    //PUT - Update an existing resource
    @PutMapping("/{id}")
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    @ApiOperation(value = "Update a Single Pessoa", response = PessoaTeste.class)
    @ApiResponses(value = {
        @ApiResponse(code = 200, message = "Successfully"),
        @ApiResponse(code = 424, message = "Houve um problema com a Mãe"),})
    public ResponseEntity<PessoaTeste> update(@RequestBody @Valid PessoaTeste pessoa, @PathVariable Long id) {
        LOG.info("update" + id);

        Optional<PessoaTeste> optional = pessoas.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        PessoaTeste existente = optional.get();

        try {
            return ResponseEntity.ok(pessoas.update(pessoa, existente));
        } catch (UnexpectedRollbackException ex) {
            if (ex.getMostSpecificCause() instanceof SQLIntegrityConstraintViolationException) {
                if (ex.getMessage().contains("FK_Pessoa_PessoaMae")) {
                    return ResponseEntity
                            .status(HttpStatus.FAILED_DEPENDENCY)
                            .build();
                }
            }
        }

        return ResponseEntity.status(500).build();

    }

}

//200 - SUCESS
//404 - RESOURCE NOT FOUND
//400 - BAD REQUEST
//201 - CREATED
//401 - UNAUTHORIZED
//415 - UNSUPPORTED TYPE - Representation not supported for the resource
//500 - SERVER ERROR
// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
//https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg
// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
GET 	/device-management/devices : Get all devices
POST 	/device-management/devices : Create a new device

GET 	/device-management/devices/{id} : Get the device information identified by "id"
PUT 	/device-management/devices/{id} : Update the device information identified by "id"
DELETE	/device-management/devices/{id} : Delete device by "id"*/
