package com.alessiojr.gamepoints.resource;

import com.alessiojr.gamepoints.model.Medalha;
import java.util.List;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alessiojr.gamepoints.service.MedalhaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.util.Optional;
import javax.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;

@CrossOrigin
@RestController
@RequestMapping("/gamepoints/medalha")
@Api(value = "/gamepoints/medalha", tags = "Medalha Controller",  description = "Operações sobre a Entidade Medalha")
public class MedalhaController {
    
    private static final Logger log = LoggerFactory.getLogger(MedalhaController.class);
    
    @Autowired
    private MedalhaService medalhas;
    
    //GET - Read all resource
    @ApiOperation(value = "View a list of available medals - Veja uma lista de medalhas disponíveis", response = List.class)
    @GetMapping(params = {"nome", "pagina"})
    public Page<Medalha> findAll(
            @RequestParam String nome,
            @RequestParam Integer pagina) {
        return this.medalhas.findByNomeContaining(nome, pagina);
    }
     //GET - Read a resource
    @ApiOperation(value = "Find by Id - Encontre uma medalha por id", response = List.class)
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Medalha> findById(@PathVariable Long id){
        log.info("Passei no /games/medalhas/" + id);
        Optional<Medalha> medalha = medalhas.findById(id);
       
        if (!medalha.isPresent()){
           return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(medalha.get());
    }
    
    @ApiOperation(value = "Delete - Exclua uma medalha", response = List.class)
    @DeleteMapping("/{id}")
//  @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity delete(@PathVariable Long id) {
        log.info("Passei no Delete /gamepoints/medalha" + id);
        Optional<Medalha> medalha = medalhas.findById(id);
        if (!medalha.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            medalhas.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }
    
    @PutMapping("/{id}")
    @ApiOperation(value = "Update - Altere uma medalha", response = List.class)
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Medalha> update(@RequestBody @Valid Medalha medalha, @PathVariable Long id) {
        Optional<Medalha> optional = medalhas.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        Medalha existente = optional.get();
        return ResponseEntity.ok(medalhas.update(medalha, existente));
    }
        
    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Create a Single Medal - Crie uma única medalha", response = Medalha.class)
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Medalha criada com sucesso!"),
        @ApiResponse(code = 400, message = "Já existe uma medalha com os mesmos dados!"),
        @ApiResponse(code = 401, message = "Você não está autorizado a visualizar o recurso!"),
        @ApiResponse(code = 403, message = "Acessar o recurso que você estava tentando acessar é proibido!"),
        @ApiResponse(code = 404, message = "O recurso que você estava tentando acessar não foi encontrado!")
    })
   
    public ResponseEntity<Medalha>  create(
    @ApiParam(value = "Medalha com elementos Nome, Descrição, Categoria, Tipo, Regra, Data e Pontos") @Valid @RequestBody Medalha medalha) {
    return ResponseEntity
            .status(HttpStatus.CREATED)
            .body(medalhas.insert(medalha));
    }
}


//200 - SUCESS
//404 - RESOURCE NOT FOUND
//400 - BAD REQUEST
//201 - CREATED
//401 - UNAUTHORIZED
//415 - UNSUPPORTED TYPE - Representation not supported for the resource
//500 - SERVER ERROR

// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
//https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg

// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
GET 	/device-management/devices : Get all devices
POST 	/device-management/devices : Create a new device

GET 	/device-management/devices/{id} : Get the device information identified by "id"
PUT 	/device-management/devices/{id} : Update the device information identified by "id"
DELETE	/device-management/devices/{id} : Delete device by "id"*/