package com.alessiojr.influenzer.resource;

import com.alessiojr.influenzer.model.Campanha;
import com.alessiojr.influenzer.service.CampanhaService;
import java.util.List;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.util.Optional;
import javax.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;


/**
 *
 * @author Pedro Paulo
 */
@CrossOrigin
@RestController
@RequestMapping("/influenzer/campanha")
@Api(value = "/influenzer/campanha", tags = "Campanha Controler",  description = "Operações sobre a Entidade Campanha")
public class CampanhaController {
    
    private static final Logger log = LoggerFactory.getLogger(CampanhaController.class);
    
    @Autowired
    private CampanhaService campanhas;

    //get
    @ApiOperation(value = "Lista de campanhas disponíveis", response = List.class)
    @GetMapping(params = {"nome", "pagina"})
    public Page<Campanha> findAll(
            @RequestParam String nome,
            @RequestParam Integer pagina) {
        return this.campanhas.findByNomeContaining(nome, pagina);
    }
    
    //get id
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Campanha> findById(@PathVariable Long id){
       log.info("Passei no /influenzer/campanha" + id);
       Optional<Campanha> campanha = campanhas.findById(id);
       if (!campanha.isPresent()){
          return ResponseEntity.notFound().build();
       }
       return ResponseEntity.ok(campanha.get());
    }
    
    //delete
    @DeleteMapping("/{id}")
    public ResponseEntity delete(@PathVariable Long id) {
        log.info("Passei no Delete /influenzer/campanha" + id);
        Optional<Campanha> campanha = campanhas.findById(id);
        if (!campanha.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        try {
            campanhas.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }
        return ResponseEntity.noContent().build();
    }
    
    //put
    @PutMapping("/{id}")
    public ResponseEntity<Campanha> update(@RequestBody @Valid Campanha campanha, @PathVariable Long id) {
        Optional<Campanha> optional = campanhas.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        Campanha existente = optional.get();
        return ResponseEntity.ok(campanhas.update(campanha, existente));
    }
     
    //create
    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Criar uma Campanha", response = Campanha.class)
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Successfully Created"),
        @ApiResponse(code = 400, message = "Já existe uma campanha para este com a mesma descrição"),
        @ApiResponse(code = 401, message = "You are not authorized to view the resource"),
        @ApiResponse(code = 403, message = "Accessing the resource you were trying to reach is forbidden"),
        @ApiResponse(code = 404, message = "The resource you were trying to reach is not found"),
        @ApiResponse(code = 406, message = ""),
        @ApiResponse(code = 500, message = "")
    })
    public ResponseEntity<Campanha>  create(
        @ApiParam(value = "Campanhas com elementos: Nome e Empresa") @Valid @RequestBody Campanha campanha) {
        return ResponseEntity
                .status(HttpStatus.CREATED)
                .body(campanhas.insert(campanha));
    }

    
    /*
    //get
    @ApiOperation(value = "View a list of available campaign", response = List.class)
    @GetMapping(params = {"pagina"})
    public Page<Campanha> findAll(@RequestParam Integer pagina) {
        return this.campanhas.findAll(pagina);
    }
    */
}
