package com.alessiojr.ipc.school.resource;

import com.alessiojr.ipc.school.model.Aluno;
import java.util.List;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alessiojr.ipc.school.service.AlunoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.util.Optional;
import javax.validation.Valid;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseStatus;

@CrossOrigin
@RestController
@RequestMapping("/ipc/school/aluno")
@Api(value = "/ipc/school/aluno", tags = "Aluno Controller",  description = "Operações sobre a Entidade Aluno")
public class AlunoController {
    
    private static final Logger log = LoggerFactory.getLogger(AlunoController.class);
    
    @Autowired
    private AlunoService alunos;

    //GET - Read all resource
    @ApiOperation(value = "View a list of available medals - Veja uma lista de alunos disponíveis", response = List.class)
    @GetMapping(params = {"nome", "pagina"})
    public Page<Aluno> findAll(
            @RequestParam String nome,
            @RequestParam Integer pagina) {
        return this.alunos.findByNomeContaining(nome, pagina);
    }
    
    //GET - Read a resource
    @ApiOperation(value = "Find by Id - Encontre um aluno por id", response = List.class)
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Aluno> findById(@PathVariable Long id){
        log.info("Passei no /ipc/school/aluno/" + id);
       Optional<Aluno> aluno = alunos.findById(id);
       
       if (!aluno.isPresent()){
          return ResponseEntity.notFound().build();
       }
       return ResponseEntity.ok(aluno.get());
       
    }
    
    //GET - Read a resource 
    @ApiOperation(value = "Delete - Exclua um aluno", response = List.class)
    @DeleteMapping("/{id}")
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity delete(@PathVariable Long id) {
       log.info("Passei no Delete /ipc/school/aluno/" + id);
        Optional<Aluno> aluno = alunos.findById(id);
        if (!aluno.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            alunos.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }
    
    @PutMapping("/{id}")
    @ApiOperation(value = "Update - Altere um aluno", response = List.class)
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Aluno> update(@RequestBody @Valid Aluno aluno, @PathVariable Long id) {
        Optional<Aluno> optional = alunos.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        Aluno existente = optional.get();
        return ResponseEntity.ok(alunos.update(aluno, existente));
    }
        
    @PostMapping("/")
    @ResponseStatus(HttpStatus.CREATED)
    @ApiOperation(value = "Create a Single Medal - Crie um único aluno", response = Aluno.class)
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Aluno criado com sucesso!"),
        @ApiResponse(code = 400, message = "Já existe um aluno com os mesmos dados!"),
        @ApiResponse(code = 401, message = "Você não está autorizado a visualizar o recurso!"),
        @ApiResponse(code = 403, message = "Acessar o recurso que você estava tentando acessar é proibido!"),
        @ApiResponse(code = 404, message = "O recurso que você estava tentando acessar não foi encontrado!")
    })
   
    public ResponseEntity<Aluno>  create(
    @ApiParam(value = "Aluno com elementos Nome, CPF, cidade, telefone, sexo, curso, dtnasc, matricula") @Valid @RequestBody Aluno aluno) {
    return ResponseEntity
            .status(HttpStatus.CREATED)
            .body(alunos.insert(aluno));
    }
}


//200 - SUCESS
//404 - RESOURCE NOT FOUND
//400 - BAD REQUEST
//201 - CREATED
//401 - UNAUTHORIZED
//415 - UNSUPPORTED TYPE - Representation not supported for the resource
//500 - SERVER ERROR

// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
//https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg

// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
GET 	/device-management/devices : Get all devices
POST 	/device-management/devices : Create a new device

GET 	/device-management/devices/{id} : Get the device information identified by "id"
PUT 	/device-management/devices/{id} : Update the device information identified by "id"
DELETE	/device-management/devices/{id} : Delete device by "id"*/