package com.alessiojr.ipc.submission.model;

import com.alessiojr.ipc.problemManager.model.Problema;
import com.alessiojr.ipc.competicao.model.Competicao;
import com.alessiojr.drDebug.model.Time;

import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;

import java.io.Serializable;

import java.util.Objects;

import javax.persistence.UniqueConstraint;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.ForeignKey;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Lob;
import javax.persistence.Id;

import javax.validation.constraints.NotNull;

import lombok.AllArgsConstructor;
import lombok.NoArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

/**
 * Classe Submission - Modela uma submissão de arquivos num sistema
 * de competição.
 * 
 * @author leonamtv
 * @author André Marcelino de Souza Neves
 */

@Entity
@Getter @Setter @ToString
@NoArgsConstructor 
@AllArgsConstructor
@Table(
    uniqueConstraints = {
        @UniqueConstraint( name = "UK_Submission_submitid", columnNames = "submitid" )
    }
)
@ApiModel( description = "Class representing a submission on the system." )
public class Submission implements Serializable {

    @Id
    @Column ( updatable = false, nullable = false )
    @GeneratedValue ( strategy = GenerationType.IDENTITY )
    @ApiModelProperty ( notes = "The database generated Submission ID." )
    private Long submitid;
    
    @ApiModelProperty(notes = "Id da submissão original, em caso de resubmissão.")
    private Long origsubmitid;
    
    @ManyToOne
    @NotNull
    @ApiModelProperty(notes = "Competição da qual a submissão foi realizada.")
    @JoinColumn(
        foreignKey = @ForeignKey(name = "FK_Submission_Contest"),
        name = "competicao_id_fk",
        nullable = false,
        referencedColumnName = "cid")
    private Competicao contest;

    @ManyToOne
    @ApiModelProperty(notes = "Time que submeteu.")
    @JoinColumn(
        foreignKey = @ForeignKey(name = "FK_Submission_Time"),
        name = "team_id_fk",
        nullable = false,
        referencedColumnName = "id")
    private Time team;
    
    @ManyToOne
    @ApiModelProperty(notes = "Problema submetido.")
    @JoinColumn(
        foreignKey = @ForeignKey(name = "FK_Submission_Problema"),
        name = "problem_id_fk",
        nullable = false,
        referencedColumnName = "idProblema")
    private Problema problema;

    @NotNull
    @ApiModelProperty(notes = "Id da linguagem utilizada")
    private String langid;
    
    @NotNull
    @ApiModelProperty(notes = "Tempo de submissão")
    private double submittime;
    
    @ApiModelProperty(notes = "Juiz que realizou o julgamento da submissão")
    private String judgehost;
    
    @NotNull
    @ApiModelProperty(notes = "Status da submissão. Se falso, a submissão é ignorada")
    private int valid;
    
    @ApiModelProperty(notes = "Id do rejulgamento, caso haja um")
    private Long rejudgingid;
    
    @ApiModelProperty(notes = "JSON com resultados esperados")
    private String expected_results;
    
    @ApiModelProperty(notes = "Id da submissão, caso a mesma venha de outro sistema de submissões externo")
    private String externalid;
    
    @ApiModelProperty(notes = "Resultado da submissão caso a mesma venha de outro sistema de submissão externo")
    private String externalresult;
    
    @ApiModelProperty(notes = "Ponto de entrada do programa enviado. Exemplo: main.java, main.c etc.")
    private String entry_point;
    
    @ApiModelProperty(notes = "Endereço da pasta onde os arquivos da submissão se encontram")
    private String submission_files_path;

    @ApiModelProperty(notes = "Execução finalizada")
    private boolean executed;

    @ApiModelProperty(notes = "Submissão julgada")
    private boolean judged;

    @ApiModelProperty(notes = "Sintaxe plantuml do diagrama de classes da submissão")
    @Lob
    @Column( length = 100000)
    private String plantuml_classdiagram;

    @ApiModelProperty(notes = "Sintaxe plantuml do diagrama de classes da submissão")
    @Lob
    @Column( length = 100000)
    private String plantuml_sequencediagram;

    @ApiModelProperty(notes = "JSON com resultados obtidos a partir da execução")
    @Lob
    private String executionResults;

    @ApiModelProperty(notes = "JSON com resultados das avaliações do checkstyle")
    @Lob
    private String checkStyleEvaluation;

    @Override
    public int hashCode() {
        return Objects.hash(this.submitid, this.contest.getCid(), this.team.getId(), this.problema.getIdProblema()); 
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Submission other = (Submission) obj;
        if (Double.doubleToLongBits(this.submittime) != Double.doubleToLongBits(other.submittime)) {
            return false;
        }
        if (this.valid != other.valid) {
            return false;
        }
        if (this.executed != other.executed) {
            return false;
        }
        if (this.judged != other.judged) {
            return false;
        }
        if (!Objects.equals(this.langid, other.langid)) {
            return false;
        }
        if (!Objects.equals(this.judgehost, other.judgehost)) {
            return false;
        }
        if (!Objects.equals(this.expected_results, other.expected_results)) {
            return false;
        }
        if (!Objects.equals(this.externalid, other.externalid)) {
            return false;
        }
        if (!Objects.equals(this.externalresult, other.externalresult)) {
            return false;
        }
        if (!Objects.equals(this.entry_point, other.entry_point)) {
            return false;
        }
        if (!Objects.equals(this.submission_files_path, other.submission_files_path)) {
            return false;
        }
        if (!Objects.equals(this.plantuml_classdiagram, other.plantuml_classdiagram)) {
            return false;
        }
        if (!Objects.equals(this.plantuml_sequencediagram, other.plantuml_sequencediagram)) {
            return false;
        }
        if (!Objects.equals(this.executionResults, other.executionResults)) {
            return false;
        }
        if (!Objects.equals(this.checkStyleEvaluation, other.checkStyleEvaluation)) {
            return false;
        }
        if (!Objects.equals(this.submitid, other.submitid)) {
            return false;
        }
        if (!Objects.equals(this.origsubmitid, other.origsubmitid)) {
            return false;
        }
        if (!Objects.equals(this.contest, other.contest)) {
            return false;
        }
        if (!Objects.equals(this.team, other.team)) {
            return false;
        }
        if (!Objects.equals(this.problema, other.problema)) {
            return false;
        }
        if (!Objects.equals(this.rejudgingid, other.rejudgingid)) {
            return false;
        }
        return true;
    }
}
