package com.alessiojr.ipc.submission.resource;

import com.alessiojr.ipc.submission.model.Submission;
import com.alessiojr.ipc.submission.service.SubmissionService;
import com.alessiojr.drDebug.service.FileStorageService;
import com.alessiojr.drDebug.util.Files;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Optional;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

/**
 *
 * @author leonamtv
 * @author André Marcelino de Souza Neves
 */

@CrossOrigin
@RestController
@RequestMapping("/submission")
@ApiResponses(value = { @ApiResponse(code = 201, message = "Criado com sucesso"),
        @ApiResponse(code = 400, message = "Já existe uma submissão para esta com a mesma descrição"),
        @ApiResponse(code = 401, message = "Você não é autorizado a visualizar esse recurso"),
        @ApiResponse(code = 403, message = "Acessar o recurso que você estava tentado é proibido"),
        @ApiResponse(code = 404, message = "O recurso que você estava querendo alcançar não foi achado") })
@Api(value = "/submission", tags = "Submission Controller", description = "Operações sobre a entidade submission.")
public class SubmissionController {

    @Autowired
    private SubmissionService submissions;

    @Autowired
    private FileStorageService fileStorageService;

    @ApiOperation(value = "Visualizar a lista completa de submissões", response = List.class)
    @GetMapping(params = { "query", "pagina" })
    public Page<Submission> findAllByQuery(@RequestParam Optional<String> query, @RequestParam Integer pagina) {
       return this.submissions.findByQuery(query, pagina);
    }

    @GetMapping("/{submitid}")
    @ApiOperation(value = "Visualizar uma submissão em específico", response = ResponseEntity.class)
    public ResponseEntity<Submission> findById(@PathVariable Long submitid) {
        Optional<Submission> submission = submissions.findById(submitid);
        if (!submission.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(submission.get());
    }

    @GetMapping(value = "/{submitid}/class-diagram.png", produces = MediaType.IMAGE_PNG_VALUE)
    @ApiOperation(value = "Obter o diagrama de classes respectivo a uma submissão", response = ResponseEntity.class)
    public @ResponseBody byte[] getClassDiagram(@PathVariable Long submitid, HttpServletRequest request)
            throws IOException {
        Optional<Submission> submission = submissions.findById(submitid);
        if (!submission.isPresent()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Submissão não encontrada");
        }
        String relativeFileName = submission.get().getSubmission_files_path() + "/classDiagram.png";
        String fileName = Files.getFile(relativeFileName).getAbsolutePath();
        Resource resource = fileStorageService.loadFileAsResource(fileName);
        InputStream in = resource.getInputStream();
        return IOUtils.toByteArray(in);
    }

    @GetMapping(value = "/{submitid}/class-diagram-thumbnail.png", produces = MediaType.IMAGE_PNG_VALUE)
    @ApiOperation(value = "Obter a miniatura do diagrama de classes respectivo a uma submissão", response = ResponseEntity.class)
    public @ResponseBody byte[] getClassDiagramThumbnail(@PathVariable Long submitid, HttpServletRequest request)
            throws IOException {
        Optional<Submission> submission = submissions.findById(submitid);
        if (!submission.isPresent()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Submissão não encontrada");
        }
        String relativeFileName = submission.get().getSubmission_files_path() + "/classDiagramThumbnail.png";
        String fileName = Files.getFile(relativeFileName).getAbsolutePath();
        Resource resource = fileStorageService.loadFileAsResource(fileName);
        InputStream in = resource.getInputStream();
        return IOUtils.toByteArray(in);
    }

    @GetMapping(value = "/{submitid}/sequence-diagram.png", produces = MediaType.IMAGE_PNG_VALUE)
    @ApiOperation(value = "Obter o diagrama de classe sequência respectivo a uma submissão", response = ResponseEntity.class)
    public @ResponseBody byte[] getSequenceDiagram(@PathVariable Long submitid, HttpServletRequest request)
            throws IOException {
        Optional<Submission> submission = submissions.findById(submitid);
        if (!submission.isPresent()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Submissão não encontrada");
        }
        String relativeFileName = submission.get().getSubmission_files_path() + "/sequenceDiagram.png";
        String fileName = Files.getFile(relativeFileName).getAbsolutePath();
        Resource resource = fileStorageService.loadFileAsResource(fileName);
        InputStream in = resource.getInputStream();
        return IOUtils.toByteArray(in);
    }

    @GetMapping(value = "/{submitid}/sequence-diagram-thumbnail.png", produces = MediaType.IMAGE_PNG_VALUE)
    @ApiOperation(value = "Obter a miniatura do diagrama de sequência respectivo a uma submissão", response = ResponseEntity.class)
    public @ResponseBody byte[] getSequenceDiagramThumbnail(@PathVariable Long submitid, HttpServletRequest request)
            throws IOException {
        Optional<Submission> submission = submissions.findById(submitid);
        if (!submission.isPresent()) {
            throw new ResponseStatusException(HttpStatus.NOT_FOUND, "Submissão não encontrada");
        }
        String relativeFileName = submission.get().getSubmission_files_path() + "/sequenceDiagramThumbnail.png";
        String fileName = Files.getFile(relativeFileName).getAbsolutePath();
        Resource resource = fileStorageService.loadFileAsResource(fileName);
        InputStream in = resource.getInputStream();
        return IOUtils.toByteArray(in);
    }

    @DeleteMapping("/{submitid}")
    @ApiOperation(value = "Excluir uma submissão em específico")
    public void delete(@PathVariable Long submitid) {
        this.submissions.deleteById(submitid);
    }

    @PutMapping(value = "/{submitid}")
    @ApiOperation(value = "Altera os atributos de uma submissão")
    public ResponseEntity update(@PathVariable Long submitid, @RequestBody Submission submission) {

        Optional<Submission> submissionExistente = submissions.findById(submitid);

        if (submissionExistente.isPresent()) {
            Submission sub = submissionExistente.get();
            sub.setSubmitid(submission.getSubmitid());
            sub.setContest(submission.getContest());
            sub.setEntry_point(submission.getEntry_point());
            sub.setExternalid(submission.getExternalid());
            sub.setExpected_results(submission.getExpected_results());
            sub.setExternalresult(submission.getExternalresult());
            sub.setJudgehost(submission.getJudgehost());
            sub.setLangid(submission.getLangid());
            sub.setOrigsubmitid(submission.getOrigsubmitid());
            sub.setProblema(submission.getProblema());
            sub.setRejudgingid(submission.getRejudgingid());
            sub.setSubmission_files_path(submission.getSubmission_files_path());
            sub.setSubmittime(submission.getSubmittime());
            sub.setTeam(submission.getTeam());
            sub.setValid(submission.getValid());
            submissions.save(sub);
            return ResponseEntity.ok("Submissão atualizada com sucesso");
        } else {
            return ResponseEntity.notFound().build();
        }
    }

    @PostMapping("/")
    @ApiOperation(value = "Cria uma submissão", response = Submission.class)
    @ResponseStatus(HttpStatus.CREATED)
    public Submission create(
            @ApiParam(value = "Cria submissão com atributos respectivos") @Valid @RequestBody Submission submission)
            throws ResponseStatusException {

        Optional<Submission> submissionExistente = submissions.findById(submission.getSubmitid());

        if (submissionExistente.isPresent()) {
            throw new ResponseStatusException(HttpStatus.BAD_REQUEST, "Já existe uma submissão para esta com mesmo ID");
        }

        System.out.println(submission);

//        Submission s = submissions.save(submission);

//        final String filePath = "./uploads/_submission_" + s.getSubmitid();
//
//        // TODO: Receber arquivo via upload
//        final String original = "./uploads/submission_test";
//        try {
//            Files.createRelativeFolder(filePath + "/source");
//            Files.copyDirectory(original, filePath + "/source");
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//
//        s.setExecuted(false);
//        s.setJudged(false);
//        s.setSubmission_files_path(filePath);
//        submissions.save(s);

        return submission;
    }
}
