package com.alessiojr.laboratory.resource;

import com.alessiojr.laboratory.model.ProblemaTecnico;
import java.util.List;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import java.util.Optional;
import javax.validation.Valid;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import com.alessiojr.laboratory.service.ProblemaTecnicoService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.RequestParam;

@CrossOrigin
@RestController
@RequestMapping("/laboratory/problemas")
@Api(value = "/laboratory/problemas", tags = "Problema Técnico Controler",  description = "Operações sobre a Entidade Problema Técnico")
public class ProblemaTecnicoController {
    
    private static final Logger log = LoggerFactory.getLogger(ProblemaTecnicoController.class);
    
    @Autowired
    private ProblemaTecnicoService problemas;

    //GET - Read all resource
    @ApiOperation(value = "View da lista de problemas técnicos cadastrados", response = List.class)
    @GetMapping(params = {"titulo", "pagina"})
    public Page<ProblemaTecnico> findAll(
            @RequestParam String titulo,
            @RequestParam Integer pagina) {
        return this.problemas.findByTituloContaining(titulo, pagina);
    }
    
    //GET - Read a resource
    @ApiOperation(value = "Procurar problema técnico por id", response = List.class)
    @GetMapping("/{id}")
    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<ProblemaTecnico> findById(@PathVariable Long id){
        Optional<ProblemaTecnico> problema = problemas.findById(id);
       
        if (!problema.isPresent()){
           return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(problema.get());
    }


    @ApiOperation(value = "Excluir um problema técnico", response = List.class)
    @DeleteMapping("/{id}")
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity delete(@PathVariable Long id) {
        log.info("Passei no Delete /laboratory/problemas/" + id);
        Optional<ProblemaTecnico> problema = problemas.findById(id);
        if (!problema.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            problemas.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }
    
    
    //PUT - Update an existing resource
    @ApiOperation(value = "Editar um problema técnico", response = List.class)
    @PutMapping("/{id}")
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<ProblemaTecnico> update(@RequestBody @Valid ProblemaTecnico problemaTecnico, @PathVariable Long id) {
        Optional<ProblemaTecnico> optional = problemas.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        ProblemaTecnico existente = optional.get();
        return ResponseEntity.ok(problemas.update(problemaTecnico, existente));
    }
        
    
    
    @ApiResponses(value = {
        @ApiResponse(code = 201, message = "Problema criado com sucesso"),
        @ApiResponse(code = 400, message = "Já existe um problema com esses mesmos dados"),
        @ApiResponse(code = 401, message = "Você não está autorizado a visualizar o recurso"),
        @ApiResponse(code = 403, message = "Acessar o recurso que você estava tentando acessar é proibido"),
        @ApiResponse(code = 404, message = "O recurso que você estava tentando acessar não foi encontrado")
    })
    
    @ApiOperation(value = "Criar um problema técnico", response = ProblemaTecnico.class)
    @PostMapping
    public ResponseEntity<ProblemaTecnico> insert(@RequestBody ProblemaTecnico problema) {
        try {
            return ResponseEntity
                    .status(HttpStatus.CREATED)
                    .body(problemas.insert(problema));
            
        } catch (Exception e) {
            return ResponseEntity.badRequest().build();
        }
    }
    
    public ResponseEntity<ProblemaTecnico>  create(
        @ApiParam(value = "Criar um problema id e título") @Valid @RequestBody ProblemaTecnico problemaTecnico) {
        return ResponseEntity
                .status(HttpStatus.CREATED)
                .body(problemas.insert(problemaTecnico));
	}
}


//200 - SUCESS
//404 - RESOURCE NOT FOUND
//400 - BAD REQUEST
//201 - CREATED
//401 - UNAUTHORIZED
//415 - UNSUPPORTED TYPE - Representation not supported for the resource
//500 - SERVER ERROR

// https://github.com/swagger-api/swagger-core/wiki/annotations
// https://github.com/swagger-api/swagger-core/wiki/Swagger-2.X---Annotations
//https://dzone.com/articles/spring-boot-2-restful-api-documentation-with-swagg

// Put Vs Post: https://restfulapi.net/rest-put-vs-post/
/*
GET 	/device-management/devices : Get all devices
POST 	/device-management/devices : Create a new device

GET 	/device-management/devices/{id} : Get the device information identified by "id"
PUT 	/device-management/devices/{id} : Update the device information identified by "id"
DELETE	/device-management/devices/{id} : Delete device by "id"*/