package com.alessiojr.security.resource;

import com.alessiojr.security.model.RecuperarSenha;
import com.alessiojr.security.model.Usuario;
import com.alessiojr.common.enums.Status;
import com.alessiojr.security.model.AlterarSenhaDTO;
import com.alessiojr.security.model.UsuarioDTO;
import com.alessiojr.security.service.UsuarioService;
import io.swagger.annotations.Api;
import java.io.IOException;
import java.util.Optional;
import javax.activation.FileTypeMap;
import javax.validation.Valid;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping("/usuarios")
@Api(value = "/usuarios", tags = "Usuários",  description = "Recursos Usuário")
public class UsuarioResource {

    @Autowired
    private UsuarioService usuarios;

    @GetMapping(params = {"nome", "pagina"})
    public Page<Usuario> findAll(@RequestParam String nome, @RequestParam Integer pagina) {
        return this.usuarios.findByNome(nome, pagina);
    }

    @GetMapping(params = {"nome", "status", "pagina"})
    public Page<Usuario> findByNomeAndStatus(@RequestParam String nome, @RequestParam Status status, @RequestParam Integer pagina) {
        return this.usuarios.findByNomeAndStatus(nome, status, pagina);
    }

    @PostMapping
    //@PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Usuario> insert(@RequestBody @Valid Usuario usuario) {
        return ResponseEntity
                .status(HttpStatus.CREATED)
                .body(usuarios.insert(usuario));
    }

    @GetMapping("/{id}")
    public ResponseEntity<Usuario> findById(@PathVariable Long id) {
        Optional<Usuario> usuario = usuarios.findById(id);
        if (!usuario.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok(usuario.get());
    }

    @GetMapping("/validar-login/{login}")
    public ResponseEntity<Usuario> loginValidate(@PathVariable String login) {
        Optional<Usuario> usuario = usuarios.findByLogin(login);
        if (!usuario.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok().build();
    }

    @GetMapping("/validar-email/{email}")
    public ResponseEntity<Usuario> emailValidate(@PathVariable String email) {
        Optional<Usuario> usuario = usuarios.findByEmail(email);
        if (!usuario.isPresent()) {
            return ResponseEntity.notFound().build();
        }
        return ResponseEntity.ok().build();
    }

    @PutMapping("/{id}")
//    @PreAuthorize("authentication.name == #usuario.login")
    public ResponseEntity<Usuario> update(
            @RequestBody UsuarioDTO usuario,
            @PathVariable Long id,
            @RequestParam Boolean alterarSenha) {

        Optional<Usuario> optional = usuarios.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        Usuario existente = optional.get();
        Usuario u = usuarios.update(usuario, existente, alterarSenha);
        if (u == null) {
            return ResponseEntity.badRequest().build();
        }
        return ResponseEntity.ok(u);
    }

    @PutMapping("/alterar-senha/{id}")
    public ResponseEntity<Usuario> alterarSenha(
            @RequestBody AlterarSenhaDTO alterarSenha,
            @PathVariable Long id) {

        Optional<Usuario> optional = usuarios.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        Usuario existente = optional.get();
        UsuarioDTO dto = new UsuarioDTO();

        BeanUtils.copyProperties(existente, dto);
        dto.setSenha(alterarSenha.getSenha());
        dto.setNovaSenha(alterarSenha.getNovaSenha());

        Usuario u = usuarios.update(dto, existente, Boolean.TRUE);
        if (u == null) {
            return ResponseEntity.badRequest().build();
        }
        return ResponseEntity.ok(u);
    }

    @PostMapping("/recuperar-senha/{email}")
    public ResponseEntity passwordRecover(@PathVariable String email) {
        Optional<Usuario> optional = usuarios.findByEmail(email);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        this.usuarios.generateNovaSenhaToken(optional.get());

        return ResponseEntity.ok().build();
    }

    @GetMapping("/recuperar-senha/{token}")
    public ResponseEntity<String> changeTempPassword(@PathVariable String token) {
        Optional<RecuperarSenha> optional = usuarios.findByToken(token);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        boolean status = this.usuarios.atualizaSenhaTemporaria(optional.get());
        String mensagem;
        if (status) {
            mensagem = "Sua senha foi alterada com sucesso!";
        } else {
            mensagem = "Seu link expirou. Será necessário realizar uma nova solicitação de senha!";
        }

        return ResponseEntity.ok(mensagem);
    }

    @PostMapping("/foto/{id}")
    public ResponseEntity<Usuario> updateFoto(@RequestParam MultipartFile foto, @PathVariable Long id) {
        Optional<Usuario> optional = usuarios.findById(id);
        if (!optional.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        Usuario existente = optional.get();
        return ResponseEntity.ok(this.usuarios.atualizarFoto(existente, foto));
    }

    @GetMapping("/foto/{nome}")
    public ResponseEntity<InputStreamResource> findFoto(@PathVariable String nome) throws IOException {
        try {
            Resource foto = this.usuarios.getFoto(nome);

            if (!foto.exists()) {
                return ResponseEntity.notFound().build();
            }

            return ResponseEntity
                    .ok()
                    .contentType(MediaType.valueOf(FileTypeMap.getDefaultFileTypeMap().getContentType(foto.getFile())))
                    .body(new InputStreamResource(foto.getInputStream()));
        } catch (IOException e) {
            return ResponseEntity.notFound().build();
        }
    }

    @DeleteMapping("/{id}")
    @PreAuthorize("hasAuthority('ROLE_ADMINISTRADOR')")
    public ResponseEntity<Usuario> delete(@PathVariable Long id) {
        Optional<Usuario> usuario = usuarios.findById(id);
        if (!usuario.isPresent()) {
            return ResponseEntity.notFound().build();
        }

        try {
            usuarios.deleteById(id);
        } catch (Exception e) {
            System.out.println(e.getClass().getName() + ": " + e.getMessage());
            return ResponseEntity.badRequest().build();
        }

        return ResponseEntity.noContent().build();
    }

}
