package com.alessiojr.security.service;

import com.alessiojr.account.model.Conta;
import com.alessiojr.account.repository.ContaRepository;
import com.alessiojr.security.model.RecuperarSenha;
import com.alessiojr.security.model.Usuario;
import com.alessiojr.common.enums.Status;
import com.alessiojr.security.repository.UsuarioRepository;
import com.alessiojr.security.model.UsuarioDTO;
import com.alessiojr.security.storage.UsuarioStorage;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Optional;
import java.util.Random;
import java.util.UUID;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.Resource;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.security.crypto.bcrypt.BCrypt;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

@Service
public class UsuarioService {

    @Autowired
    private UsuarioRepository<Usuario> usuarios;

    @Autowired
    private RecuperarSenhaRepository recuperarSenha;

    @Autowired
    private ContaRepository contas;
        
    @Autowired
    private UsuarioStorage storage;

    public Optional<Usuario> findById(Long idUsuario) {
        return this.usuarios.findById(idUsuario);
    }

    public Optional<Usuario> findByLogin(String login) {
        return this.usuarios.findByLogin(login);
    }

    public Optional<Usuario> findByEmail(String email) {
        return this.usuarios.findByEmail(email);
    }

    public Page<Usuario> findByNome(String nome, Integer pagina) {
        return this.usuarios.findByNome(nome, PageRequest.of(pagina, 10, Sort.by(Sort.Direction.ASC, "nome")));
    }

    public Page<Usuario> findByNomeAndStatus(String nome, Status status, Integer pagina) {
        return this.usuarios.findByNomeAndStatus(nome, status, PageRequest.of(pagina, 10, Sort.by(Sort.Direction.ASC, "nome")));
    }

    public void deleteById(Long idUsuario) {
        try {
            this.usuarios.deleteById(idUsuario);
        } catch (Exception e) {
            Optional<Usuario> op = this.findById(idUsuario);
            op.get().setStatus(Status.INATIVO);

            this.usuarios.save(op.get());
        }
    }

    public Optional<Usuario> doLogin(String login) {
        return this.usuarios.doLogin(login);
    }

    public Optional<RecuperarSenha> findByToken(String token) {
        return this.recuperarSenha.findByToken(token);
    }

    public Usuario update(UsuarioDTO usuario, Usuario existente, Boolean alterarSenha) {
        if (alterarSenha) {
            if (!BCrypt.checkpw(usuario.getSenha(), existente.getSenha())) {// compara senhas antigas
                return null; // se senhas diferentes
            }
            usuario.setSenha(new BCryptPasswordEncoder().encode(usuario.getNovaSenha())); // altera a senha
            BeanUtils.copyProperties(usuario, existente, "idUsuario", "novaSenha", "urlFoto");

        } else {
            BeanUtils.copyProperties(usuario, existente, "idUsuario", "senha", "novaSenha", "urlFoto");
        }


        return this.usuarios.save(existente);
    }

    public Usuario insert(Usuario u) {
        Usuario usuario = (Usuario) u;

        // salvar a senha utilizando o metodo bcrypt para realizar a criptografia
        usuario.setSenha(new BCryptPasswordEncoder().encode(usuario.getSenha()));
        
        return this.usuarios.save(usuario);
    }

    public void generateNovaSenhaToken(Usuario usuario) {
        String token = UUID.randomUUID().toString();

        RecuperarSenha r = new RecuperarSenha();
        r.setToken(token);
        r.setUsuario(usuario);
        r.setValidade(LocalDate.now().plusDays(2));

        String senhaTemp = this.generatePassword();
        r.setSenhaTemporaria(new BCryptPasswordEncoder().encode(senhaTemp)); //senha criptografada

        this.recuperarSenha.save(r);

        RecuperarSenha aux = new RecuperarSenha();
        aux.setSenhaTemporaria(senhaTemp); // senha não criptografada
        BeanUtils.copyProperties(r, aux, "senhaTemporaria");

    }

    public boolean atualizaSenhaTemporaria(RecuperarSenha r) {
        if (LocalDate.now().compareTo(r.getValidade()) < 1) { // se token ainda é valido
            r.getUsuario().setSenha(r.getSenhaTemporaria());
            this.usuarios.save(r.getUsuario());

            this.recuperarSenha.delete(r);
            return true;
        }
        this.recuperarSenha.delete(r);

        return false;
    }

    public Usuario atualizarFoto(Usuario usuario, MultipartFile foto) {
        if (usuario.getUrlFoto() != null) {
            this.storage.removerFoto(usuario.getUrlFoto());
        }
        usuario.setUrlFoto(this.storage.salvar(foto));
        return this.usuarios.save(usuario);
    }

    public Resource getFoto(String nome) {
        return this.storage.getFoto(nome);
    }

    private String generatePassword() {
        String senha = "";
        Random r = new Random();

        for (int i = 0; i < 6; i++) {
            if (i % 2 == 0) {
                senha += Character.toString((char) (97 + r.nextInt(26))); // gera letras minusculas
            } else {
                senha += Character.toString((char) (48 + r.nextInt(10))); // gera numeros
            }
        }

        return senha;
    }

}
